@@ -1,6 +1,6 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2009 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
 /* NXAGENT, NX protocol compression and NX extensions to this software    */
 /* are copyright of NoMachine. Redistribution and use of the present      */
@@ -9,7 +9,7 @@
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
+/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -46,6 +46,41 @@
  * initialized.
  */
 
+typedef struct
+{
+  CARD32 flags;
+  CARD32 input;
+  CARD32 initial_state;
+  CARD32 icon_pixmap;
+  CARD32 icon_window;
+  INT32  icon_x;
+  INT32  icon_y;
+  CARD32 icon_mask;
+  CARD32 window_group;
+}
+nxagentWMHints;
+
+/*
+ * This structure is compatible with 32
+ * and 64 bit library interface. It has
+ * been copied from Xatomtype.h and it's
+ * a parameter of XChangeProperty().
+ */
+
+typedef struct
+{
+  unsigned long flags;
+  long          input;
+  long          initialState;
+  unsigned long iconPixmap;
+  unsigned long iconWindow;
+  long          iconX;
+  long          iconY;
+  unsigned long iconMask;
+  unsigned long windowGroup;
+}
+nxagentPropWMHints;
+
 WindowPtr nxagentRootlessWindow = NULL;
 
 #define TOP_LEVEL_TABLE_UNIT 100
@@ -414,7 +449,8 @@ int nxagentExportProperty(pWin, property, type, format, mode, nUnits, value)
   char *propertyS, *typeS;
   Atom propertyX, typeX;
   char *output = NULL;
-  XWMHints wmHints;
+  nxagentWMHints wmHints;
+  nxagentPropWMHints propHints;
   Bool export = False;
   Bool freeMem = False;
 
@@ -470,13 +506,27 @@ int nxagentExportProperty(pWin, property, type, format, mode, nUnits, value)
   else if (strcmp(typeS, "WM_HINTS") == 0)
   {
     ClientPtr pClient = wClient(pWin);
-    wmHints = *(XWMHints*)value;
+    wmHints = *(nxagentWMHints*)value;
 
     wmHints.flags |= InputHint;
     wmHints.input = True;
 
-    output = (char*) &wmHints;
-    export  = True;
+    /*
+     * Initialize the structure used in XChangeProperty().
+     */
+
+    propHints.flags = wmHints.flags;
+    propHints.input = (wmHints.input == True ? 1 : 0);
+    propHints.initialState = wmHints.initial_state;
+    propHints.iconPixmap = wmHints.icon_pixmap;
+    propHints.iconWindow = wmHints.icon_window;
+    propHints.iconX = wmHints.icon_x;
+    propHints.iconY = wmHints.icon_y;
+    propHints.iconMask = wmHints.icon_mask;
+    propHints.windowGroup = wmHints.window_group;
+
+    output = (char*) &propHints;
+    export = True;
 
     if ((wmHints.flags & IconPixmapHint) && (wmHints.icon_pixmap != None))
     {
@@ -490,16 +540,17 @@ int nxagentExportProperty(pWin, property, type, format, mode, nUnits, value)
           nxagentSynchronizeRegion((DrawablePtr) icon, NullRegion, NEVER_BREAK, NULL);
         }
 
-        wmHints.icon_pixmap = nxagentPixmap(icon);
+        propHints.iconPixmap = nxagentPixmap(icon);
       }
       else
       {
-        wmHints.flags &= ~IconPixmapHint;
+        propHints.flags &= ~IconPixmapHint;
 
         #ifdef WARNING
-        fprintf(stderr, "nxagentExportProperty: WARNING! Failed to look up icon pixmap %lx from hint "
+        fprintf(stderr, "nxagentExportProperty: WARNING! Failed to look up icon pixmap %x from hint "
                     "exporting property %s type %s on window %p.\n",
-                        wmHints.icon_pixmap, propertyS, typeS, (void*)pWin);
+                        (unsigned int) wmHints.icon_pixmap, propertyS, typeS,
+                            (void *) pWin);
         #endif
       }
     }
@@ -511,16 +562,17 @@ int nxagentExportProperty(pWin, property, type, format, mode, nUnits, value)
 
       if (icon)
       {
-        wmHints.icon_window = nxagentWindow(icon);
+        propHints.iconWindow = nxagentWindow(icon);
       }
       else
       {
-        wmHints.flags &= ~IconWindowHint;
+        propHints.flags &= ~IconWindowHint;
 
         #ifdef WARNING
-        fprintf(stderr, "nxagentExportProperty: WARNING! Failed to look up icon window %lx from hint "
+        fprintf(stderr, "nxagentExportProperty: WARNING! Failed to look up icon window %x from hint "
                     "exporting property %s type %s on window %p.\n",
-                        wmHints.icon_window, propertyS, typeS, (void*)pWin);
+                        (unsigned int) wmHints.icon_window, propertyS, typeS,
+                            (void *) pWin);
         #endif
       }
     }
@@ -532,16 +584,17 @@ int nxagentExportProperty(pWin, property, type, format, mode, nUnits, value)
 
       if (icon)
       {
-        wmHints.icon_mask = nxagentPixmap(icon);
+        propHints.iconMask = nxagentPixmap(icon);
       }
       else
       {
-        wmHints.flags &= ~IconMaskHint;
+        propHints.flags &= ~IconMaskHint;
 
         #ifdef WARNING
-        fprintf(stderr, "nxagentExportProperty: WARNING! Failed to look up icon mask %lx from hint "
+        fprintf(stderr, "nxagentExportProperty: WARNING! Failed to look up icon mask %x from hint "
                     "exporting property %s type %s on window %p.\n",
-                        wmHints.icon_mask, propertyS, typeS, (void*)pWin);
+                        (unsigned int) wmHints.icon_mask, propertyS, typeS,
+                            (void *) pWin);
         #endif
       }
     }
@@ -553,16 +606,17 @@ int nxagentExportProperty(pWin, property, type, format, mode, nUnits, value)
 
       if (window)
       {
-        wmHints.window_group = nxagentWindow(window);
+        propHints.windowGroup = nxagentWindow(window);
       }
       else
       {
-        wmHints.flags &= ~WindowGroupHint;
+        propHints.flags &= ~WindowGroupHint;
 
         #ifdef WARNING
-        fprintf(stderr, "nxagentExportProperty: WARNING! Failed to look up window group %lx from hint "
+        fprintf(stderr, "nxagentExportProperty: WARNING! Failed to look up window group %x from hint "
                     "exporting property %s type %s on window %p.\n",
-                        wmHints.window_group, propertyS, typeS, (void*)pWin);
+                        (unsigned int) wmHints.window_group, propertyS, typeS,
+                            (void *) pWin);
         #endif
       }
     }
@@ -654,7 +708,7 @@ int nxagentExportProperty(pWin, property, type, format, mode, nUnits, value)
   {
     #ifdef TEST
     fprintf(stderr, "nxagentExportProperty: WARNING! Ignored ChangeProperty "
-                "on %swindow %lx property %s type %s nUnits %ld format %d\n",
+                "on %swindow %x property %s type %s nUnits %ld format %d\n",
                     nxagentWindowTopLevel(pWin) ? "toplevel " : "",
                         nxagentWindow(pWin), validateString(propertyS), validateString(typeS),
                             nUnits, format);
@@ -683,7 +737,7 @@ void nxagentImportProperty(Window window,
   WindowPtr pWin;
   Bool import = False;
   Bool freeMem = False;
-  XWMHints wmHints;
+  nxagentWMHints wmHints;
 
   typedef struct {
       CARD32 state;
@@ -797,7 +851,7 @@ void nxagentImportProperty(Window window,
   }
   else if (strcmp(typeS, "WM_HINTS") == 0)
   {
-    wmHints = *(XWMHints*)buffer;
+    wmHints = *(nxagentWMHints*)buffer;
     output = (char*) &wmHints;
     import = True;
 
@@ -815,8 +869,9 @@ void nxagentImportProperty(Window window,
 
         #ifdef WARNING
         fprintf(stderr, "nxagentImportProperty: WARNING! Failed to look up remote icon "
-                    "pixmap %ld from hint importing property [%ld] type %s on window %p.\n",
-                        wmHints.icon_pixmap, (long int) property, typeS, (void *) pWin);
+                    "pixmap %d from hint importing property [%ld] type %s on window %p.\n",
+                        (unsigned int) wmHints.icon_pixmap, (long int) property,
+                            typeS, (void *) pWin);
         #endif
       }
     }
@@ -835,8 +890,9 @@ void nxagentImportProperty(Window window,
 
         #ifdef WARNING
         fprintf(stderr, "nxagenImportProperty: WARNING! Failed to look up remote icon "
-                    "window %lx from hint importing property [%ld] type %s on window %p.\n",
-                         wmHints.icon_window, (long int) property, typeS, (void *) pWin);
+                    "window %x from hint importing property [%ld] type %s on window %p.\n",
+                         (unsigned int) wmHints.icon_window,
+                             (long int) property, typeS, (void *) pWin);
         #endif
       }
     }
@@ -855,8 +911,8 @@ void nxagentImportProperty(Window window,
 
         #ifdef WARNING
         fprintf(stderr, "nxagentImportProperty: WARNING! Failed to look up remote icon "
-                    "mask %lx from hint importing property [%ld] type %s on window %p.\n",
-                          wmHints.icon_mask, (long int) property, typeS, (void *) pWin);
+                    "mask %x from hint importing property [%ld] type %s on window %p.\n",
+                          (unsigned int) wmHints.icon_mask, (long int) property, typeS, (void *) pWin);
         #endif
       }
     }
@@ -875,8 +931,9 @@ void nxagentImportProperty(Window window,
 
         #ifdef WARNING
         fprintf(stderr, "nxagentImportProperty: WARNING! Failed to look up remote window "
-                    "group %lx from hint importing property [%ld] type %s on window %p.\n",
-                          wmHints.window_group, (long int) property, typeS, (void *) pWin);
+                    "group %x from hint importing property [%ld] type %s on window %p.\n",
+                          (unsigned int) wmHints.window_group,
+                              (long int) property, typeS, (void *) pWin);
         #endif
       }
     }
